import 'dart:isolate';

import 'package:indra/indra.dart';
import 'package:indra/src/output/console_output.dart';
import 'package:indra/src/output/isolate_output.dart';
import 'package:indra/src/runner.dart';

Map<String, String> params;

Map<String, String> setup(SendPort outputPort, List<String> args, {Map<String, String> defaultParams = const {}}) {
  output = outputPort != null ? new IsolateOutput(outputPort) : new ConsoleOutput();
  params = defaultParams;
  args.forEach((a) {
    var keyValue = a.split('=');
    if (keyValue.length != 2) {
      throw new ArgumentError('$a\nUsage: parameter=value');
    }
    params[keyValue[0]] = keyValue[1];
  });
  if (params.containsKey('workingDir')) {
    Context.changeDir(params['workingDir']);
  } else {
    Context.changeDir(Shell.workingDirectory);
  }
  Shell.rootDirectory = Shell.workingDirectory;
  return params;
}

Runner bootstrap(SendPort outputPort, List<String> args, {Map<String, String> defaultParams = const {}}) =>
    Runner(setup(outputPort, args, defaultParams: defaultParams));

String requiredParam(Map<String, String> params, String name) {
  if (!params.containsKey(name) || params[name].isEmpty) {
    output.showError('Missing required param "$name"');
    throw new TaskFailed();
  }
  return params[name];
}

typedef void Script(Map<String, String> params);

class Runner {
  Map<String, String> _params;

  Runner(this._params);

  run(Script script) async {
    try {
      await script(_params);
    } on TaskFailed catch (e) {
      output.showError(e.message);
      throw e;
    } catch (e, stacktrace) {
      output.showError(e.toString(), stacktrace.toString());
      throw e;
    }
  }
}
